#!/usr/bin/python
#
# Actions handler for the client stuff
#
# $Id: client.py,v 1.21 2002/09/03 10:01:17 gafton Exp $

import os
import sys
import string
import rpm
import xmlrpclib
import fnmatch

# up2date modules
from up2date_client import up2date
from up2date_client import config
from up2date_client import up2dateAuth
from up2date_client import rpmUtils
from up2date_client import rpcServer
from up2date_client import up2dateLog

cfg = config.initUp2dateConfig()
log = up2dateLog.initLog()

# mark this module as acceptable
__rhnexport__ = [
    'update_for_errata',
    'update_packages',
    'remove_packages',
    'client',
]

# action version we understand
ACTION_VERSION = 2 

# Where do we keep the CA certificate for RHNS?
# The servers we're talking to need to have their certs
# signed by one of these CA.
rhns_ca_cert = "/usr/share/rhn/RHNS-CA-CERT"

###
# MAIN CLIENT CLASS
###
class client:
    def __init__(self):
        self.functions = []
        self.functions.append("refresh")
        self.functions.append("refresh_rpmlist")
        self.functions.append("new_systemid")
	self.functions.append("update_packages")
        self.functions.append("up2date")
        self.functions.append("updates_message")
        self.functions.append("remove_packages")
        self.functions.append("update_up2date_config")
        self.functions.append("get_up2date_config")
        self.functions.append("update_for_errata")
       
    def __getattr__(self, attr):
	if attr == "__rhnexport__":
	    return self.functions
	else:
	    return getattr(self, attr)

    def get_function(self, function):
        if function in self.functions:
            return getattr(self, function)
        return None

    def get_functions(self):
	return self.functions
 
    # push again the list of rpm packages to the server
    def refresh_rpmlist(self):
	s = rpcServer.getServer()
 
        log.log_debug("Called refresh_rpmlist")
           
        ret = None
        try:
            s.registration.update_packages(up2dateAuth.getSystemId(),
                                           rpmUtils.getInstalledPackageList())
        except:
            print "ERROR: refreshing remote package list for System Profile"
            return xmlrpclib.Fault(-3, "Error refreshing package list")
        return 0


    # resync hardware information with the server profile
    def refresh_hardware(self):
        from up2date_client import hardware
        # read all hardware in
        hardwareList = hardware.Hardware()
        
	s = rpcServer.getServer()
	
        log.log_debug("Called refresh_hardware")

        try:
            s.registration.refresh_hw_profile(up2dateAuth.getSystemId(),
                                              hardwareList)
        except:
            print "ERROR: sending hardware database for System Profile"
            return xmlrpclib.Fault(-4, "Error refreshing system hardware")
        return 0

    
    def refresh(self, database):
        """Refresh the RHN image of what is available on this server"""
        if type(database) != type(""):
            return xmlrpclib.Fault(-8, "Invalid refresh action requested")
        action = "refresh_%s" % string.lower(database)
        func = None
        if hasattr(self, action):
            func = getattr(self, action)
        if func == None:
            return xmlrpclib.Fault(-9, "Invalid refresh action called")
        return func()


    def new_systemid(self, id):
        """We have been given a new certificate - install it"""

        log.log_debug("Called new_systemid")

        path = cfg["systemIdPath"]
        try:
            os.rename(path, path + ".bak")
        except:
            print "ERROR: could not back up %s" % path
            return xmlrpclib.Fault(-1, "Could not back up old SystemID")

        f = open(path, "w")
        f.write(id)
        f.close()
        try:
            os.chmod(path, 0600)
        except:
            print "ERROR: could not adjust permissions to 0600 on %s" % path
            return xmlrpclib.Fault(-2, "Could not adjust permissions on "\
                                   "new SystemID")
        return 0

    def update_up2date_config(self, configdict):
        """Invoke this to change the ondisk configuration of up2date"""
        log.log_debug("called update_up2date_config")

        if type(configdict) != type({}):
            return xmlrpclib.Fault(-5, "Invalid arguments passed to function")

        unknownparams = []
        for param in configdict.keys():
            if cfg.has_key(param):
                cfg.set(param, configdict[param])
            else:
                unknownparams.append(param)
                
        if len(unknownparams):
            return unknownparams

        cfg.save()

        return 0

    def get_up2date_config(self):
        """Reterieve the current configuration of up2date"""
        log.log_debug("called get_up2date_config")
        ret = {}
        for k in cfg.keys():
            ret[k] = cfg[k]
        return ret
    

    def remove_packages(self, pkgList):
        """We have been told that we should remove packages"""
        def rpmCallback(what, amount, total, hdr, path):
            if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
                global fd
                fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                                   hdr['name'],
                                                   hdr['version'],
                                                   hdr['release'],
                                                   hdr['arch'])
                try:
                     fd = os.open(fileName, os.O_RDONLY)
                except:
                    print "Error opening %s for reading" % fileName
                    return -1

                return fd
            elif what == rpm.RPMCALLBACK_INST_START:
                fileName = "%s-%s-%s.%s.rpm" % (hdr['name'],
                                                hdr['version'],
                                                hdr['release'],
                                                hdr['arch'])
            elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
                os.close(fd)

            elif what == rpm.RPMCALLBACK_INST_PROGRESS:
                pass
            elif what == rpm.RPMCALLBACK_UNINST_STOP:
                pass
            
        if type(pkgList) != type([]):
            return xmlrpclib.Fault(-5, "Invalid arguments passed to function")
        
        log.log_debug("Called remove_packages", pkgList)

        # FIXME: figure out the error handling
        try:
            up2date.removePackages(pkgList, rpmCallback)
        except up2dateErrors.DependencyError, e:
            # just return the list of deps if we failed
            return e
        except up2dateErrors.RpmRemoveError, e:
            # if we later need finer grained info, we can
            return xmlrpclib.Fault(-6, "%s" % e)
        except up2dateErrors.RpmRemoveSkipListError, e:
            return xmlrpclib.Fault(-6, "%s could not be removed because "\
                                   "it was on the skip list" % e)

        # if were succesful, return "4"
        return 4
    
    def update_packages(self, pkgList):        
        """We have been told that we should retrieve/install packages"""

        def rpmCallback(what, amount, total, hdr, path):
            if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
                global fd
                fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                                   hdr['name'],
                                                   hdr['version'],
                                                   hdr['release'],
                                                   hdr['arch'])
                try:
                     fd = os.open(fileName, os.O_RDONLY)
                except:
                    print "Error opening %s for reading" % fileName
                    return -1

                return fd
            elif what == rpm.RPMCALLBACK_INST_START:
                fileName = "%s-%s-%s.%s.rpm" % (hdr['name'],
                                                hdr['version'],
                                                hdr['release'],
                                                hdr['arch'])
            elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
                os.close(fd)

            elif what == rpm.RPMCALLBACK_INST_PROGRESS:
                pass
            elif what == rpm.RPMCALLBACK_UNINST_STOP:
                pass

        if type(pkgList) != type([]):
            return xmlrpclib.Fault(-5, "Invalid arguments passed to function")
        
        log.log_debug("Called update_packages", pkgList)

        return self.batchRun(0, pkgList)

    def batchRun(self, onlyList, pkgList):
        from up2date_client import wrapper
        ret = wrapper.batchRun(onlyList, [], 0, fromDaemon = 1,
                               actionPkgs = pkgList)
        if ret < 0:
             return xmlrpclib.Fault(-6, "Failed: packages requested "\
                                    "are no longer available")
        elif ret > 0:
             return xmlrpclib.Fault(-6, "Failed: packages requested "\
                                    "raised  dependency problems")
        elif ret == 0:
             # status code for "completed"
             return 4
        return 0
 
    def updates_message(self, message):
        pass


# export these functions at the class and module level
# to avoid any compability issues
# FIXME: remove these later
c = client()
def update_for_errata(errataid):
	return c.update_for_errata(errataid)

def update_packages(pkgList):
	return c.update_packages(pkgList)

def remove_packages(pkgList):
	return c.remove_packages(pkgList)

