#!/usr/bin/python

import string
import os
import sys
import unittest

import coverage



#class HtmlTestRunner(unittest.TextTestRunner):
#    def __init__(self, verbosity=1):
#        unittest.TestTextRunner.__init__(verbosity=verbosity)##
#
#    def run(self, test):

class MetaTestResult(unittest.TestResult):
    def __init__(self):
        self.startTests = []
        self.stopTests = []

    def startTest(self, test):
        for st in self.startTests:
            apply(st, (self,test))
        self.testsRun  = self.testsRun + 1
        
    def stopTest(self, test):
        for st in self.stopTests:
            apply(st, (self, test))

            
class TimedTestResult:
    def __init__(self):
        self.times = []
    
    def startTest(self, test):
        self.starttime = time.time()

    def stopTest(self, test):
        self.stoptime = time.time()
        self.runtime = self.stoptime - self.starttime
        
class TimedTextTestResult(TimedTestResult):    
    # somehow need to make the methods "mixin", not sure yet how...
    # maybe make addSuccess call all self.__addSuccess* methods with
    # same args as addSuccess? That would let us create a result class
    # that can at least store all the different info bits
    def addSuccess(self, test):
        self.times.append(test, self.runTime)





class TestCase(unittest.TestCase):
    def __init__(self, methodName):
        self.callType = "default"
        unittest.TestCase.__init__(self,methodName)

    def defaultTestResult(self):
        #print "foobar"
        print "self.callType: %s" % self.callType
        if self.callType == "default":
            return unittest._TextTestResult(stream,description,verbosity)
        elif self.callType == "time":
            return TimedTextTestResult(stream,description,verbosity)
        elif self.callType == "profile":
            return unittest.TextTestResult(stream,description,verbosity)
        

    def __call__(self, result=None):
        print self.callType
        print type(self.callTypes)
        print self.callTypes['default']
        return apply(self.__call__, result = self.defaultTestResult() )
        


class TestProgram:
    """A command-line program that runs a set of tests; this is primarily
       for making test modules conveniently executable.
    """
    USAGE = """\
Usage: %(progName)s [options] [test] [...]
                                                                                                                                        
Options:
  -h, --help       Show this message
  -v, --verbose    Verbose output
  -q, --quiet      Minimal output
                                                                                                                                        
Examples:
  %(progName)s                               - run default set of tests
  %(progName)s MyTestSuite                   - run suite 'MyTestSuite'
  %(progName)s MyTestCase.testSomething      - run MyTestCase.testSomething
  %(progName)s MyTestCase                    - run all 'test*' test methods
                                               in MyTestCase
"""
    def __init__(self, module='__main__', defaultTest=None,
                 argv=None, testRunner=None, testLoader=unittest.defaultTestLoader, covmodule=None):
        if type(module) == type(''):
            self.module = __import__(module)
            for part in string.split(module,'.')[1:]:
                self.module = getattr(self.module, part)
        else:
            self.module = module
        if argv is None:
            argv = sys.argv
        self.verbosity = 1
        self.coverage = 0
        self.defaultTest = defaultTest
        self.testRunner = testRunner
        self.testLoader = testLoader
        self.progName = os.path.basename(argv[0])
        self.covmodule = covmodule
        self.parseArgs(argv)
        self.runTests()
                                                                                                                                        
    def usageExit(self, msg=None):
        if msg: print msg
        print self.USAGE % self.__dict__
        sys.exit(2)

    def parseArgs(self, argv):
        import getopt
        try:
            options, args = getopt.getopt(argv[1:], 'hHvqc',
                                          ['help','verbose','quiet', 'coverage'])
            for opt, value in options:
                if opt in ('-h','-H','--help'):
                    self.usageExit()
                if opt in ('-q','--quiet'):
                    self.verbosity = 0
                if opt in ('-v','--verbose'):
                    self.verbosity = 2
                if opt in ('-c', '--coverage'):
                    self.coverage = 1
            if len(args) == 0 and self.defaultTest is None:
                self.test = self.testLoader.loadTestsFromModule(self.module)
                return
            if len(args) > 0:
                self.testNames = args
            else:
                self.testNames = (self.defaultTest,)
            self.createTests()
        except getopt.error, msg:
            self.usageExit(msg)
                                                                                                                                        
    def createTests(self):
        self.test = self.testLoader.loadTestsFromNames(self.testNames,
                                                       self.module)

    def saveHtml(self, buf, filename, directory):
        import os
        basename = os.path.basename(filename)

        f = open("%s/%s.html" % (directory, basename), "w")
        f.write(buf)
        f.close()

    def genHtml(self, filename, statements, missing, coverage, directory):
        import cov2html
        title = "Coverage info for %s" % filename

        totallen = len(statements) + len(missing)
        covered = int((float(len(statements))/float(totallen)*100))
        nc = 100 - covered

        header = "<table><tr><td>%s</td><td>%s%%</td></tr></table>" % (filename, covered)


        buf = cov2html.processString(coverage, header, title)
        self.saveHtml(buf, filename, directory)


                                                                                                                                        
    def runTests(self):
        if self.testRunner is None:
            self.testRunner = unittest.TextTestRunner(verbosity=self.verbosity)
        if self.coverage:
            coverage.erase()
            coverage.start()
            reload(self.covmodule)
        result = self.testRunner.run(self.test)
        if self.coverage:
            coverage.stop()
        m = sys.modules.values()
        if self.coverage:
            coverage.the_coverage.report(self.covmodule, ignore_errors=1, show_missing=0 )
            annotedoutput = coverage.the_coverage.annotateToString(self.covmodule, ignore_errors=1)
            (filename, statements, missing, blip) = coverage.analysis(self.covmodule)
#        print "f: %s" % filename
#        print "statements: %s" % statements
#        print "missing: %s" % missing
#        print "blip: %s" % blip

        #print annotedoutput
        if self.coverage:
            self.genHtml(filename, statements, missing, annotedoutput, directory="test-results/")

        sys.exit(not result.wasSuccessful())
                                                                                                                                        
main = TestProgram
                                                                                                                                        
