#!/usr/bin/python


import unittest
import time
import sys
import string
import os
import dumper

class TimedTestResult(unittest.TestResult):
    def __init__(self):
        unittest.TestResult.__init__(self)

    def startTest(self, test):
        self.starttime = time.time()
        self.testsRun  = self.testsRun + 1
    
    def stopTest(self, test):
        self.stoptime = time.time()
        self.runtime = self.stoptime - self.starttime
        
class _TimedTextTestResult(TimedTestResult):    
    def addSuccess(self, test):
        #timeString = "SetupTime: %s\n" % self.setupTime
        timeString = timeString + "RunTime: %s\n" % self.runTime
        #timeString = timeString + "TearDownTime: %s\n\n" % self.tearDownTime
        #dumper.dump(self.stream)
        self.stream.write("ok\n")
        self.stream.write(timeString)

    

testCase = "default"
stream = sys.stderr
description = ""
verbosity = 1


class RhnTestCase(unittest.TestCase):
    def __init__(self, methodName):
        try:
            self.__testMethodName = methodName
            testMethod = getattr(self, methodName)
            self.__testMethodDoc = testMethod.__doc__
        except AttributeError:
            raise ValueError, "no such test method in %s: %s" % \
                  (self.__class__, methodName)
        unittest.TestCase.__init__(self,methodName)


    def defaultTestResult(self):
        #print "foobar"
        print "self.callType: %s" % self.callType
        if self.callType == "default":
            return unittest._TextTestResult(stream,description,verbosity)
        elif self.callType == "time":
            return _TimedTextTestResult(stream,description,verbosity)
        elif self.callType == "profile":
            return unittest.TextTestResult(stream,description,verbosity)

    def __call__(self, result=None):
        print self.callType
        print type(self.callTypes)
        print self.callTypes['default']
        return apply(self.__call__, result = self.defaultTestResult() )
        



class TimedTextTestRunner(unittest.TextTestRunner):
    def __init__(self, stream=sys.stderr, descriptions=1, verbosity=1):
        unittest.TextTestRunner.__init__(self, stream=stream, descriptions=descriptions, verbosity=verbosity)
        print "blippy: %s" % self.verbosity
    def _makeResult(self):
        blip =  _TimedTextTestResult(self.stream, self.descriptions, self.verbosity)
        return blip


defaultTestLoader = unittest.TestLoader()


class RhnTestProgram(unittest.TestProgram):
    """A command-line program that runs a set of tests; this is primarily
    for making test modules conveniently executable.
    """
    USAGE = """\
Usage: %(progName)s [options] [test] [...]

Options:
  -h, --help       Show this message
  -v, --verbose    Verbose output
  -q, --quiet      Minimal output

Examples:
  %(progName)s                               - run default set of tests
  %(progName)s MyTestSuite                   - run suite 'MyTestSuite'
  %(progName)s MyTestCase.testSomething      - run MyTestCase.testSomething
  %(progName)s MyTestCase                    - run all 'test*' test methods
                                               in MyTestCase
"""
    def __init__(self, module='__main__', defaultTest=None,
                 argv=None, testRunner=None, testLoader=defaultTestLoader):
        if type(module) == type(''):
            self.module = __import__(module)
            for part in string.split(module,'.')[1:]:
                self.module = getattr(self.module, part)
        else:
            self.module = module
        if argv is None:
            argv = sys.argv
        self.verbosity = 1
        self.profile = None
        self.time = None
        self.defaultTest = defaultTest
        self.testRunner = testRunner
        self.testLoader = testLoader
        self.progName = os.path.basename(argv[0])
        self.parseArgs(argv)
        self.runTests()

    def usageExit(self, msg=None):
        if msg: print msg
        print self.USAGE % self.__dict__
        sys.exit(2)

    def parseArgs(self, argv):
        import getopt
        try:
            options, args = getopt.getopt(argv[1:], 'hHvq',
                                          ['help','verbose','quiet'
                                           'profile', 'time'])
            for opt, value in options:
                print opt, value
                if opt in ('-h','-H','--help'):
                    self.usageExit()
                if opt in ('-q','--quiet'):
                    self.verbosity = 0
                if opt in ('-v','--verbose'):
                    self.verbosity = 2
                if opt in ['--profile']:
                    self.profile = 1
                if opt in ['--time']:
                    self.time = 1

            global testCase
            testCase = "default"
            if self.time:
                self.testRunner = TimedTextTestRunner(verbosity=self.verbosity)
                testCase = "time"
            if self.profile:
                self.testRunner = ProfileTextTestRunner(verbosity=self.verbosity)


            if len(args) == 0 and self.defaultTest is None:
                self.test = self.testLoader.loadTestsFromModule(self.module)
                return
            if len(args) > 0:
                self.testNames = args
            else:
                self.testNames = (self.defaultTest,)
            self.createTests()
        except getopt.error, msg:
            self.usageExit(msg)
            
 
    def createTests(self):
        self.test = self.testLoader.loadTestsFromNames(self.testNames,
                                                       self.module)

    def runTests(self):
        if self.testRunner is None:
            self.testRunner = unittest.TextTestRunner(verbosity=self.verbosity)
        result = self.testRunner.run(self.test)
        sys.exit(not result.wasSuccessful())

main = RhnTestProgram

