#!/usr/bin/python

import sys
import string
import rpm
sys.path.append("../")

import rollbacks
import up2dateErrors

# lots of useful util methods for building/tearing down
# test enviroments...
import testutils

import unittest

import gc

def dump_garbage():
    """
    show us what's the garbage about
    """
        
    # force collection
    sys.stderr.write("\nGARBAGE:\n")
    gc.collect()

    print "\nGARBAGE OBJECTS:"
    for x in gc.garbage:
        s = str(x)
        if len(s) > 80: s = s[:80]
        sys.stderr.write("%s \n %s\n" % (type(x), s))

def write(blip):
    sys.stdout.write("\n|%s|\n" % blip)


# everything in this class should be happy with
# a empty set of rollbacks, though some test will
# fail indicating there was no data to test with
class TestRollbacks(unittest.TestCase):
    def setUp(self):
        self.rb = rollbacks.Rollback()

    def testGetLatestTid(self):
        "Verify that testGetLatestTid doesnt explode..."
        res = self.rb.getLatestTid()
        #write(res)
        # just testing to see that it doesn't splode...

    def testGetTidsSinceForever(self):
        "Verify that getTidSince(0) works..."
        t = 0
        res = self.rb.getTidsSince(t)
#        write(res)

    def testGetTidsSinceFlyingCars(self):
        "Verify that getTidsSince(epoch of 2038) works..."
        t = 2147483647
        res = self.rb.getTidsSince(t)
        assert res == []

    def testGetTidsSinceNow(self):
        "Verify that getTidsSince(_right_.... NOW!) works..."
        import time
        t = time.time()
        res = self.rb.getTidsSince(t)
        assert res == []
        

    def testGetTransactionsData(self):
        "Verify that getTransactionData doesnt explode..."
        res = self.rb.getTransactionsData()
        #write(res)
        # again, just checking for 'splosions...

    def testGetUndoInfo(self):
        "Verify that getUndoInfo doesnt explode..."
        try:
            res = self.rb.getUndoInfo()
        except up2dateErrors.NoRollbacksToUndoError:
            # I'm cool with that...
            pass
        #write(res)
        # again, just checking for 'splosions...


    def testGetTidInfo(self):
        "Verify that getTidInfo doesn't explode..."
        tid = self.rb.getLatestTid()
        if not tid:
            self.fail("No transactions to test with")
        res = self.rb.getTidInfo(tid)
        # again, just checking for 'splosions...

    def testGetTidInfoInvalidTid(self):
        "Verify getTidInfo with invalid tid..."
        # tid are also unix time stamps... if you actually
        # have system with a rpm transaction from 1970, wtf
        # are you even bothering to run tests...
        tid = 37
        try:
            res = self.rb.getTidInfo(tid)
        except KeyError:
            pass
        else:
            self.fail("Expected an KeyError")

    def testPreviewUndo(self):
        "Verify that previewUndo doesnt explode"
        res = self.rb.previewUndo()
#        write(res)

    def testRemoveStaleTransaction(self):
        "Verify that removeStaleTransaction handles invalid tid"
        try:
            res = self.rb.removeStaleTransaction(37)
        except KeyError:
            pass
        else:
            self.fail("Expected a KeyError")

    def testGetPackages(self):
        "Verify that getPackages doesn't explode..."
        tid = self.rb.getLatestTid()
        if not tid:
            self.fail("No Transactions to test with")
        res = self.rb.getPackages(tid)

    def testGetPackagesInvalidTid(self):
        "Verify that getPackages(invalid Tid) raises a KeyError..."
        try:
            res = self.rb.getPackages(37)
        except KeyError:
            pass
        else:
            self.fail("Expected a KeyError")

    def testKeys(self):
        "Verify that Rollback.keys doesnt explode..."
        res = self.rb.keys()

    def testValues(self):
        "Verify that Rollback.items doesnt explode..."
        res = self.rb.values()

    def testHas_key(self):
        "Verify that Rollback.has_key with invalid key returns None."
        res = self.rb.has_key("37")
        assert res == 0

    def testShowTransaction(self):
        "Verify that Rollback.showTransaction returns a formatted string..."
        tid = self.rb.getLatestTid()
        res = self.rb.showTransaction(tid)
            
    def testShowTransactionInvalidTid(self):
        "Verify that Rollback.showTransaction(invalid_tid) returns None..."
        res = self.rb.showTransaction(37)
        assert res == None


# should we inherit? or maybe not by default and
# mixin if we need to?
# _TestRollbacksData is the name since we dont want to
# automagically run this class...

class _TestRollbacksData(TestRollbacks):
    setupflag = 0
    def __init__(self, *args):
        TestRollbacks.__init__(self,*args)
    
    def _realSetup(self, testname):
#        rpm.addMacro("_dbpath",testutils.DBPATH)
#        repackagedir =  "/tmp/testrepackage" 
#        rpm.addMacro("_repackage_dir", repackagedir)
        
        # setup stock data...
#        self.setupExpectedData()

        self.rb = rollbacks.Rollback()
        
        if TestRollbacksData1.setupflag != 0:
            return
        
        testutils.createDataDirs()
        testutils.rebuildRpmDatabase(testname)
        testutils.rebuildRepackageDir(testname)
        TestRollbacksData1.setupflag = 1

        
    # overwrite this to load the approriate datainfo of course
    def setUp(self):
        pass
        #self._realSetup("data1-i386")

    # make the data attributes of the object so we can use
    # the same names with different data sets...
    # overload this to load default data 
    def setupExpectedData(self):
        pass
    
    def tearDown(self):
        del self.rb


    def testPreviewUndoValidate(self):
        "Validate the preview info returned from Rollback.previewUndo"
        pass

    def testGetLatestTidValidate(self):
        "Validate the data returned from Rollback.getLatestTid"
        pass
    
    def testGetTidsSinceForeverValidate(self):
        "Validate Rollback.getTidsSince(0)"
        pass

    def testGetTransactionsDataValidate(self):
        "Validate Rollback.getTransactionData"
        pass

    def testGetUndoInfoValidate(self):
        "Validate the data returned from Rollback.getUndoInfo"
        pass
    def testGetTidInfoValidate(self):
        "Validate the data returned from Rollback.getTidInfo"
        pass

    def testGetPackagesValidate(self):
        "Validate the data returned from Rollback.getPackages"
        pass
    
    def testKeysValidate(self):
        "Validate the data returned from Rollback.keys"
        pass

    def testValuesValidate(self):
        "Validate the data returned from Rollback.values"
        pass
    
    def testShowTransactionValidate(self):
        "Validate the return values of Rollback.showTransactionValidate"
        pass
    

        
        

class TestRollbacksData1(_TestRollbacksData):
    def setUp(self):
        self._realSetup("data1-i386")
        dump_garbage()

    def setupExpectedData(self):
        self.undoinfo = {'tid': 1050355912,
                         'added_packages':[('tar', '1.13.25', '11', '', 'i386', '+')],
                         'removed_packages': [('tar', '1.13.25', '10', '', 'i386', '-')]}
        
        self.expectedPreviewUndo = """install time: Mon Apr 14 17:31:52 2003\t tid:1050355912
\t\t[-] tar-1.13.25-10:
\t\t[+] tar-1.13.25-11:"""
        self.showTransactionString = """install time: Mon Apr 14 17:31:52 2003\t tid:1050355912\n\t\t[-] tar-1.13.25-10:\n\t\t[+] tar-1.13.25-11:"""
        self.latestTid = 1050355912
        self.tidsSinceForever = [1050355912, 1050354930]
        self.transactionData = {'1050355912':
                                {'tid': 1050355912,
                                 'removed': [('tar', '1.13.25', '10', '', 'i386', '-')],
                                 'added': [('tar', '1.13.25', '11', '', 'i386', '+')]}
                                , '1050354930':
                                {'tid': 1050354930,
                                 'removed': [('zebra', '0.93a', '8', '', 'i386', '-')],
                                 'added': [('zebra', '0.93b', '1', '', 'i386', '+')]}}

        self.getTidInfoResults = {'tid': 1050355912,
                                  'added_packages': [('tar', '1.13.25', '11', '', 'i386', '+')],
                                  'removed_packages': [('tar', '1.13.25', '10', '', 'i386', '-')]}

        self.getPackagesInfo = []
        self.keysInfo = [1050355912, 1050354930]
        
if __name__ == "__main__":
    import gc
    gc.enable()
    gc.set_debug(gc.DEBUG_LEAK)
    import sys
    unittest.main(argv=sys.argv)
    dump_garbage()
    
