#!/usr/bin/python



import sys
import shutil
#sys.path.append("../")
from up2date_client import up2date
from up2date_client import up2dateErrors

import testconfig
import unittest


def write(blip):
    sys.stdout.write("\n%s\n" % blip)


# FIXME: with the new code, this will need to include
# setting up a sources list

class TestHasBadSignature(unittest.TestCase):
    def setUp(self):
        import testutils
        testutils.setupConfig("fc3-badrpms")
        self.unsignedPackageName = "unsigned-1.1-1.1.noarch.rpm"
        self.unsignedPkg = ['unsigned', '1.1', '1.1', '', 'noarch', '123123', 'directory-testrpms']

        self.signedPackageName = "signed-1.1-1.1.noarch.rpm"
        self.signedPkg = ['signed', '1.1', '1.1', '', 'noarch', '123123123',  'directory-testrpms']

        
        self.signedMungedHeaderPackageName = "signed-munged-header-1.1-1.1.noarch.rpm"
        self.signedMungedHeaderPkg = ['signed-munged-header', '1.1', '1.1', '', 'noarch', '123123123',  'directory-testrpms']

        self.unsignedMungedHeaderPackageName = "unsigned-munged-header-1.1-1.1.noarch.rpm"
        self.unsignedMungedHeaderPkg = ['unsigned-munged-header', '1.1', '1.1', '', 'noarch', '123123123',  'directory-testrpms']
        
        self.signedMungedPayloadPackageName = "signed-munged-payload-1.1-1.1.noarch.rpm"
        self.signedMungedPayloadPkg = ['signed-munged-payload', '1.1', '1.1', '', 'noarch', '123123123',  'directory-testrpms']
        self.pkgPath = testconfig.testpackagepath

    def testUnsignedPackage(self):
        "Verify that unsigned packages return 1 indicating they failed"
        shutil.copy("%s/%s" % (self.pkgPath, self.unsignedPackageName), "/var/spool/up2date/%s" %  self.unsignedPackageName)
        res = up2date.hasBadSignature(self.unsignedPkg)
        write("\n\nres: %s\n\n" % res)
        self.assertEqual(res, 1)

    def testSignedPackage(self):
        "Verify that a properly singed package verifies correctly"
        shutil.copy("%s/%s" % (self.pkgPath, self.signedPackageName), "/var/spool/up2date/%s" %  self.signedPackageName)
        res = up2date.hasBadSignature(self.signedPkg) 
        self.assertEqual(res, 0)

    def testSignedMungedHeader(self):
        "Verify that a properly singed package with a modified header fails correctly"
        shutil.copy("%s/%s" % (self.pkgPath, self.signedMungedHeaderPackageName),
                    "/var/spool/up2date/%s" %  self.signedMungedHeaderPackageName)
        try:
            res = up2date.hasBadSignature(self.signedMungedHeaderPkg)
        except up2dateErrors.RpmError:
            pass
        else:
            self.fail("Expected an RpmError")

    def testUnsignedMungedHeader(self):
        "Verify that a unsinged package with a modified header fails correctly"
        shutil.copy("%s/%s" % (self.pkgPath,self.unsignedMungedHeaderPackageName),
                    "/var/spool/up2date/%s" %  self.unsignedMungedHeaderPackageName)
        try:
            res = up2date.hasBadSignature(self.unsignedMungedHeaderPkg)
        except up2dateErrors.RpmError, e:
            pass
        else:
            self.fail("Expected an RpmError")

    def testSignedMungedPayload(self):
        "Verify that a properly signed package with a modified payload fails correctly"
        shutil.copy("%s/%s" % (self.pkgPath, self.signedMungedPayloadPackageName),
                    "/var/spool/up2date/%s" %  self.signedMungedPayloadPackageName)
        try:
            res = up2date.hasBadSignature(self.signedMungedPayloadPkg)
        except up2dateErrors.RpmError, e:
            pass
        else:
            self.fail("Expected an RpmError")
def main():
    import myunittest
    myunittest.main(argv=["-v", "-v"], covmodule=up2date)


if __name__ == "__main__":
    main()
