#!/usr/bin/python -u
#
# Part of the up2date upgrader, class handling the interface with
# the Red Hat Network database, XML RPC calls, registration,
# downloads,  etc.
#
# veillard@redhat.com
#
"""Implements a class handling the interaction with the Red Hat Network
   this includes the database access, registration and update of 
   version and profile informations, rpm sources downloads etc."""

import sys
import posix
from socket import gethostname
import xmlrpclib
sys.path.append("..")
import rpmSource
import rhnHTTPlib
import up2date
import upLog


debug = 0

class rhnInterface:
    """This is the class handling the interaction with the Red Hat Network
       this includes the database access, registration and update of 
       version and profile informations, rpm sources downloads etc."""
    def __init__(self, Server = None, SystemIdFile = None, \
                 username = None, passwd = None, email = None, \
		 Proxy = None, proxyUser = None, proxyPasswd = None, \
		 storageDir = None, packageDir = None):
        
	#
	# Certainly requires some cleanup to be realigned with
	# up2date, especially for all config options.
	#
	if Server == None:
	    self.server = "http://beta.rhns.redhat.com/XMLRPC"
#           TODO use up2date conf
#	    self.server = "http://beta.rhns.redhat.com/XMLRPC"
        else:
	    self.server = Server

        self.proxy = Proxy
        self.proxyUser = proxyUser
        self.proxyPasswd = proxyPasswd

	if SystemIdFile == None:
	    self.systemidFile = "systemid"
#           TODO use up2date conf
#           self.systemidFile = "/etc/sysconfig/rhn/systemid"
        else:
	    self.systemidFile = SystemIdFile

        self.hostname = gethostname()
#       TODO use up2date conf
        self.username = posix.environ['USERNAME']
#       TODO use up2date conf
	self.email = self.username+"@redhat.com"
#       TODO use up2date conf
	self.org_id = ""
	self.org_passwd = ""

        if packageDir == None:
#       TODO use up2date conf
#	    packageDir = "/mnt/redhat/beta/roswell/i386/RedHat/RPMS/"
	    packageDir = "/mnt/redhat/dist/7.1/i386/RedHat/RPMS/"
	up2date.cfg.writeEntry("packageDir", packageDir);
	self.packageDir = packageDir
        if storageDir == None:
#       TODO use up2date conf
	    storageDir = "/mnt/redhat/tmp/up2date"
	up2date.cfg.writeEntry("storageDir", storageDir);
	self.storageDir = storageDir
	try:
	    up2date.cfg.save()
	except:
	    upLog.error("Warning: could not save configuration")
	
	self.headerSource = None
	self.packageSource = None
	self.listPackageSource = None
	self.xmlserver = None
	self.getserver = None
	self.systemId = None
	self.authToken = None
	self.channels = None
	self.version = None
	self.BUFFER_SIZE = rpmSource.BUFFER_SIZE
	self.downloadCallback = None
	self.installCallback = None

	upLog.debug("rhnInterface initialized")

    #
    # Default progress callback
    #
    def defaultCallback(self, count, origsize, speed = 0, secs = 0):
	if count == 1 and origsize == 1:
	    upLog.progress("-- already loaded")
	else:
	    upLog.progress("-- %s bytes of %s: %s KB/s, %s secs remaining" % 
	          (count, origsize, speed, secs))

    #
    # TODO: provide accessors to all tunable parameters
    #
    def storageDirGet(self):
        return self.storageDir
    def packageDirGet(self):
        return self.packageDir
    def downloadCallbackGet(self):
        if self.downloadCallback == None:
	    return self.defaultCallback
        return self.downloadCallback
    def installCallbackGet(self):
        if self.installCallback == None:
	    return self.defaultCallback
        return self.installCallback

    #
    # Get a Package
    #
    def getPackage(self, pkg):
        if self.packageSource == None:
	    return None

        upLog.debug("getting %s-%s-%s package" % (pkg[0], pkg[1], pkg[2]))

        downloadCallback = self.downloadCallback
	# TODO: uses a upLog callback
	if debug:
	    if downloadCallback == None:
	        downloadCallback = self.defaultCallback
	ret = self.packageSource.getPackage(pkg, \
	                     progressCallback = downloadCallback)
        
	upLog.debug("packageSource.getPackage: completed")
	return ret

    #
    # Get a Header
    #
    def getHeader(self, pkg):
        if self.headerSource == None:
	    return None

        upLog.debug("getting %s-%s-%s header" % (pkg[0], pkg[1], pkg[2]))

        downloadCallback = self.downloadCallback
	# TODO: uses a upLog callback
	if debug:
	    if downloadCallback == None:
	        downloadCallback = self.defaultCallback
	hdr, type = self.headerSource.getHeader(pkg, \
	                     progressCallback = downloadCallback)

	# Shall we return (hdr, type), i.e. is the type needed to
	# process further down ???
	return hdr
        
    #
    # Register the current system to the RHN server
    #
    # Returns 0 in case of success -1 otherwise
    #
    def registerSystem(self, Server = None, username = None, passwd = None):
        # TODO
	return -1
        
    #
    # Connect and login into the RHN server
    #
    # Returns 0 in case of success -1 otherwise
    #
    def login(self, Server = None, username = None, passwd = None):
        #
	# Connect to the XMLRPC server
	#
        self.xmlserver = xmlrpclib.Server(self.server)

	#
	# try to read the systeid, otherwise register the system
	#
        try:
	    f = open(self.systemidFile, "r")
	    self.systemId = f.read()
	    f.close()
	    upLog.debug("read systemid from %s" % self.systemidFile)
        except:
	    upLog.error("Could not open %s, you need to register first" %
	                self.systemidFile)
	    self.registerSystem()

	if self.systemId == None:
	    self.xmlserver.close()
	    upLog.error("Failed to register to %s" % self.server)
	    return -1

	#
	# Okay Logn now
	#
	try:
	    self.authToken = self.xmlserver.up2date.login(self.systemId)
	except:
	    self.xmlserver.close()
	    self.xmlserver = None
	    self.channels = None
	    upLog.error("Failed to login onto %s" % self.server)
	    return -1

	self.channels = self.authToken["X-RHN-Auth-Channels"]
	self.getserver = rhnHTTPlib.Server(self.server, headers=self.authToken)

	self.headerSource = rpmSource.initHeaderSources(self.authToken,	\
	                   self.server, self.server, self.proxy, 	\
			   self.proxyUser, self.proxyPasswd)
	self.packageSource = rpmSource.initPackageSources(self.authToken,\
	                   self.server, self.server, self.proxy, 	\
			   self.proxyUser, self.proxyPasswd)
	self.listPackageSource =					\
	                   rpmSource.initListPackageSources(self.authToken,\
	                   self.server, self.server, self.proxy, 	\
			   self.proxyUser, self.proxyPasswd)

	upLog.debug("Connected and logged into %s" % self.server)
	upLog.debug("Channels: %s" % self.channels)
	return 0

    #
    # Tell the RHN server we are switching version, get and store the new
    # system ID needed for further processing, and refresh the channels
    # list
    #
    # Returns 0 in case of success -1 otherwise
    #
    def changeVersion(self, version):
        if version == None:
	    return -1
        if self.channels == None or self.systemId == None:
	    upLog.debug("changeVersion: not connected")
	    return -1

	try:
	    f = open(self.systemidFile, "w")
	except:
	    upLog.error("Could not write to %s" % self.systemidFile)
	    return -1

	newSystemId = None
	try:
	    newSystemId = self.xmlserver.registration.upgrade_version( \
	                          self.systemId, version)
	except:
	    upLog.error("RHN refused upgrade to %s" % version)
	    f.write(self.systemId);
	    f.close
	    return -1

	if newSystemId == None:
	    upLog.error("RHN refused upgrade to %s" % version)
	    f.write(self.systemId);
	    f.close
	    return -1

        #
	# save the new system ID
	#
	self.systemId = newSystemId
	f.write(self.systemId);
	f.close
	
	try:
	    self.getserver.close()
	except:
	    self.getserver = None

	#
	# Okay re-login and update login informations
	#
	try:
	    self.authToken = self.xmlserver.up2date.login(self.systemId)
	except:
	    self.xmlserver.close()
	    self.xmlserver = None
	    self.channels = None
	    upLog.error("Failed to login onto %s" % self.server)
	    return -1

	self.channels = self.authToken["X-RHN-Auth-Channels"]
	self.getserver = rhnHTTPlib.Server(self.server, headers=self.authToken)

	upLog.debug("Upgrade to %s and reconnection successful" % (version))
	upLog.debug("Channels: %s" % self.channels)


    #
    # Get the full lists of packages from the given subscribed channels
    #
    # Returns the list of packages or None in case of error
    #
    def getList(self, channelNo = 0):
        if self.channels == None or self.systemId == None:
	    upLog.debug("getList: not connected")
	    return None
	try:
	    size, fd = self.getserver.listPackages(    \
	          self.channels[channelNo][0], self.channels[channelNo][1])
	except:
	    upLog.error("getList: failed to receive package list from server")
	    return None

        #
	# Hum should this way of tunnelling XMLRPC through a GET
	# be encapsulated, probably TODO
	#
	x = ""
	while size > 0:
	    buffer = fd.read(min(self.BUFFER_SIZE, size))
	    if len(buffer) == 0:
	        break
	    size = size - len(buffer)
	    x = x + buffer
	fd.close()
	fullLists, foo = xmlrpclib.loads(x)

	list = fullLists[0]

	upLog.debug("getList: got %s packages from %s" % \
		      (len(list), self.channels[channelNo][0]))

        return list

#
# Small test routine to do basic tests
#
def main():
    global debug
    interf = None

    args = sys.argv[1:]
    while args:
        if args[0] == '--login' or args[0] == '-login':
	    if interf == None:
		interf = rhnInterface()
	    if interf == None:
		sys.exit(-1)

            interf.login()

	    print "Channels:", interf.channels

        elif args[0] == '--list' or args[0] == '-list':
	    if interf == None:
		interf = rhnInterface()
	    if interf == None:
		sys.exit(-1)

            interf.login()
            list = interf.getList()

	    print "List:", list

        elif args[0] == '--version' or args[0] == '-version':
	    args = args[1:]
	    upLog.debug("change to version", args[0])
	    if interf == None:
		interf = rhnInterface()
	    if interf == None:
		sys.exit(-1)

            interf.login()
	    interf.changeVersion(args[0])

	    print "Channels:", interf.channels

        elif args[0] == '--verbose' or args[0] == '-v':
	    debug = debug + 1

        else:
            sys.stderr.write('unknown argument '+ args[0] + '\n')
	    sys.stderr.write('use --check name version realease or -v\n')
	    sys.exit(-1)
        args = args[1:]


if __name__ == "__main__":
    main()
