#!/usr/bin/python -u
#
# Part of the up2date upgrader, class handling the logs and feedback
#
# veillard@redhat.com
#
"""Implements a module and a class used for logging the state
   changes, debug msg, error msg and the display part of the RPM
   callbacks."""

import sys

debugLevel = 0
loggerlist = None
loggerdefault = None

class defaultLogger:
    """This is the default class used for logging the state changes,
       debug msg, error msg and the display part of the RPM callbacks."""
    def __init__(self):
        global debugLevel

        #
	# Nothing major here, should allow to init from a file
	#
        self.debugLevel = debugLevel
	if debugLevel:
	    print "default logger initialized"

    #
    # Log an error message
    #
    def debugLevel(self, value):
        if value == None:
	    return

        self.debugLevel = value

    #
    # Log an error message
    #
    def error(self, msg):
        if msg == None:
	    return

        print "%s" % msg

    #
    # Log a warning message
    #
    def warning(self, msg):
        if msg == None:
	    return

	print "%s" % msg

    #
    # Log a debug message
    #
    def debug(self, msg, level = 1):
        if msg == None:
	    return

        if level <= self.debugLevel:
	    print "%s" % msg

    #
    # Log a general progress message
    #
    def progress(self, msg):
        if msg == None:
	    return

	print "%s" % msg


#
# Initialize
#
def init():
    global loggerlist
    global loggerdefault
    
    if loggerlist != None:
        return

    loggerlist = []
    loggerdefault = defaultLogger()
    loggerlist.append(loggerdefault)

#
# Add an extra logger
#
def addLogger(logger):
    global loggerlist
    if loggerlist == None:
        loggerlist = []
    loggerlist.append(logger)
    
#
# Remove the default logger
#
def disableDefaults(logger):
    global loggerlist
    global loggerdefault
    if loggerdefault == None:
	return
    if loggerlist == None:
	return

    loggerlist.delete(loggerdefault)
    loggerdefault = None
    
#
# Log an error message
#
def error(msg):
    global loggerlist
    if loggerlist == None:
        init()
    for logger in loggerlist:
        logger.error(msg)

#
# Log a debug message
#
def debug(msg, level = 1):
    global loggerlist
    if loggerlist == None:
        init()
    for logger in loggerlist:
        logger.debug(msg, level)
#
# Log a warning message
#
def warning(msg):
    global loggerlist
    if loggerlist == None:
        init()
    for logger in loggerlist:
        logger.warning(msg)

#
# Log a general progress message
#
def progress(msg):
    global loggerlist
    if loggerlist == None:
        init()
    for logger in loggerlist:
        logger.progress(msg)

